/*
 * 
 * The Seasar Software License, Version 1.1
 *
 * Copyright (c) 2003-2004 The Seasar Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or 
 * without modification, are permitted provided that the following 
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer in the documentation and/or other materials provided 
 *    with the distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgement:  
 *    "This product includes software developed by the 
 *    Seasar Project (http://www.seasar.org/)."
 *    Alternately, this acknowledgement may appear in the software
 *    itself, if and wherever such third-party acknowledgements 
 *    normally appear.
 *
 * 4. Neither the name "The Seasar Project" nor the names of its
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission of 
 *    the Seasar Project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE SEASAR PROJECT 
 * OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL,SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.seasar.util.configuration;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.Reader;
import java.io.Writer;

import org.apache.commons.configuration.AbstractFileConfiguration;
import org.apache.commons.configuration.ConfigurationException;
import org.seasar.util.configuration.ognl.ConfigurationPropertyAccessor;

/**
 * qAhLg`̃t@CݒǂݍConfigurationłB
 * 
 * @author suga
 * @author Koichi Kobayashi
 */
public class HereDocumentConfiguration extends AbstractFileConfiguration {
    protected static final String LINE_SEP = System
            .getProperty("line.separator");
    protected static final int LINE_SEP_LENGTH = LINE_SEP.length();

    /*
     * OGNL łConfigurationɃANZX邽߂̃NX[h܂B
     */
    static {
        ConfigurationPropertyAccessor.class.getName();
    }

    /**
     * CX^X\z܂B<br>
     * <code>get<var>Type</var>(String key)</code>\bhŃvpeBɃANZXꍇ
     * key ȂΗOX[悤ɐݒ肵܂B
     *
     */
    public HereDocumentConfiguration() {
        setThrowExceptionOnMissing(true);
    }

    /**
     * qAhLgǂݍ݁AConfigurationɐݒ肵܂B
     * 
     * @param reader
     *            Xg[
     * @throws ConfigurationException
     *             Xg[̓ǂݍݒɃG[ꍇɃX[܂
     */
    public void load(final Reader reader)
            throws ConfigurationException {
        try {
            final BufferedReader br = new BufferedReader(reader);
            final StringBuffer sb = new StringBuffer();
            String key = null;
            String endMark = null;
            String line;
            while ((line = br.readLine()) != null) {
                if (endMark == null) {
                    line = line.trim();
                    if (line.startsWith("#")) {
                        continue;
                    }

                    final int index = line.indexOf("<<");
                    if (index > 0) {
                        key = line.substring(0, index).trim();
                        endMark = line.substring(index + 2).trim();
                    }
                }
                else if (line.equals(endMark)) {
                    sb.setLength(sb.length() - LINE_SEP_LENGTH);
                    addProperty(key, new String(sb));
                    key = null;
                    endMark = null;
                    sb.setLength(0);
                }
                else {
                    sb.append(line);
                    sb.append(LINE_SEP);
                }
            }
        }
        catch (final IOException e) {
            throw new ConfigurationException(
                    "Could not load configuration from input stream.", e);
        }
    }

    /**
     * ݃T|[gĂ܂B
     */
    public void save(final Writer writer) throws ConfigurationException {
        throw new UnsupportedOperationException();
    }
}