/*
 * 
 * The Seasar Software License, Version 1.1
 *
 * Copyright (c) 2003-2004 The Seasar Project. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or 
 * without modification, are permitted provided that the following 
 * conditions are met:
 *
 * 1. Redistributions of source code must retain the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above 
 *    copyright notice, this list of conditions and the following 
 *    disclaimer in the documentation and/or other materials provided 
 *    with the distribution.
 *
 * 3. The end-user documentation included with the redistribution,
 *    if any, must include the following acknowledgement:  
 *    "This product includes software developed by the 
 *    Seasar Project (http://www.seasar.org/)."
 *    Alternately, this acknowledgement may appear in the software
 *    itself, if and wherever such third-party acknowledgements 
 *    normally appear.
 *
 * 4. Neither the name "The Seasar Project" nor the names of its
 *    contributors may be used to endorse or promote products derived 
 *    from this software without specific prior written permission of 
 *    the Seasar Project.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR 
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR 
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE SEASAR PROJECT 
 * OR ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, 
 * INCIDENTAL,SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES 
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS 
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING 
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF 
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package org.seasar.util.configuration;

import java.io.InputStream;
import java.io.Reader;
import java.io.Writer;

import javax.xml.parsers.SAXParser;
import javax.xml.parsers.SAXParserFactory;

import org.apache.commons.configuration.AbstractFileConfiguration;
import org.seasar.framework.util.SAXParserFactoryUtil;
import org.seasar.framework.xml.SaxHandler;
import org.seasar.framework.xml.SaxHandlerParser;
import org.seasar.framework.xml.TagHandler;
import org.seasar.framework.xml.TagHandlerContext;
import org.seasar.framework.xml.TagHandlerRule;
import org.seasar.util.configuration.ognl.ConfigurationPropertyAccessor;
import org.xml.sax.Attributes;
import org.xml.sax.InputSource;

/**
 * J2SE5.0œꂽXMLvpeBhLgɗގXMLt@CݒǂݍConfigurationłB <br>
 * XMLt@Cɂ͎̕^錾KvłB
 * 
 * <pre>
 *  &lt;!DOCTYPE properties PUBLIC &quot;-//SEASAR//DTD S2Util-Text-Env//EN&quot; &quot;http://www.seasar.org/dtd/s2util-text-env.dtd&quot;&gt;
 * </pre>
 * 
 * @author Koichi Kobayashi
 */
public class XMLConfiguration extends AbstractFileConfiguration {
    public static final String PUBLIC_ID = "-//SEASAR//DTD S2Util-Configuration//EN";
    public static final String DTD_PATH = "org/seasar/util/configuration/s2-util-configuration.dtd";

    /*
     * OGNL łConfigurationɃANZX邽߂̃NX[h܂B
     */
    static {
        ConfigurationPropertyAccessor.class.getName();
    }

    /**
     * CX^X\z܂B <br>
     * <code>get<var>Type</var>(String key)</code> \bhŃvpeBɃANZXꍇ key
     * ȂΗOX[悤ɐݒ肵܂B
     */
    public XMLConfiguration() {
        setThrowExceptionOnMissing(true);
    }

    /**
     * oCgXg[ݒ[h܂B <br>
     * X[p[NX̂̃\bh̓vbgtH[ftHg̃GR[fBO Xg[쐬Đݒ[h܂B
     * ̓XMLt@C̃[hɂ͕sK؂ł邽߂ɃI[o[ChĂ܂B
     * 
     * @param is
     *            oCgXg[
     */
    public void load(final InputStream is) {
        load(new InputSource(is));
    }

    /**
     * Xg[ݒ[h܂B
     * 
     * @param reader
     *            Xg[
     */
    public void load(final Reader reader) {
        load(new InputSource(reader));
    }

    /**
     * ̓\[Xݒ[h܂B
     * 
     * @param is
     *            ̓\[X
     */
    public void load(final InputSource is) {
        final SAXParserFactory factory = SAXParserFactoryUtil.newInstance();
        factory.setValidating(true);

        final SAXParser saxParser = SAXParserFactoryUtil.newSAXParser(factory);
        final SaxHandler handler = new SaxHandler(new XMLEnvironmentRule());
        handler.registerDtdPath(PUBLIC_ID, DTD_PATH);

        final SaxHandlerParser parser = new SaxHandlerParser(handler, saxParser);
        parser.parse(is);
    }

    /**
     * ݃T|[gĂ܂B
     */
    public void save(final Writer writer) {
        throw new UnsupportedOperationException();
    }

    class XMLEnvironmentRule extends TagHandlerRule {
        public XMLEnvironmentRule() {
            addTagHandler("entry", new EntryTagHandler());
        }
    }

    class EntryTagHandler extends TagHandler {
        public void start(final TagHandlerContext context, final Attributes attributes) {
            context.push(attributes.getValue("key"));
        }

        public void end(final TagHandlerContext context, final String body) {
            if (body != null) {
                final String value = body.trim();
                if (value.length() > 0) {
                    final String key = (String) context.pop();
                    addProperty(key, body.trim());
                }
            }
        }
    }
}